@@ -37,101 +37,107 @@
 
 - (id) initWithDevice: (void *)device
 {
-  WIN_INTERN *win = (WIN_INTERN *)GetWindowLong((HWND)device, GWL_USERDATA);
-  HDC         hDC = GetDC((HWND)device);
+  if (self)
+    {
+      WIN_INTERN *win = (WIN_INTERN *)GetWindowLong((HWND)device, GWL_USERDATA);
+      HDC         hDC = GetDC((HWND)device);
 
-  // Save/set initial state...
-  gsDevice = device;
-  _surface = NULL;
+      // Save/set initial state...
+      gsDevice = device;
+      _surface = NULL;
   
-  if (hDC == NULL)
-    {
-      NSWarnMLog(@"Win32CairoSurface line: %d : no device context", __LINE__);
+      if (hDC == NULL)
+	{
+	  NSWarnMLog(@"Win32CairoSurface line: %d : no device context", __LINE__);
       
-      // And deallocate ourselves...
-      DESTROY(self);
-    }
-  else
-    {
-      // Create the cairo surfaces...
-      // NSBackingStoreRetained works like Buffered since 10.5 (See apple docs)...
-      // NOTE: According to Apple docs NSBackingStoreBuffered should be the only one
-      //       ever used anymore....others are NOT recommended...
-      if (win && (win->type == NSBackingStoreNonretained))
-        {
-          // This is the raw DC surface...
-          _surface = cairo_win32_surface_create(hDC);
-
-          // Check for error...
-          if (cairo_surface_status(_surface) != CAIRO_STATUS_SUCCESS)
-            {
-              // Output the surface create error...
-              cairo_status_t status = cairo_surface_status(_surface);
-              NSWarnMLog(@"surface create FAILED - status: %s\n", cairo_status_to_string(status));
-              
-              // Destroy the initial surface created...
-              cairo_surface_destroy(_surface);
-              
-              // And deallocate ourselves...
-              DESTROY(self);
-            }
-        }
+	  // And deallocate ourselves...
+	  DESTROY(self);
+	}
       else
-        {
-          NSSize csize = [self size];
-          
-          // This is the raw DC surface...
-          cairo_surface_t *window = cairo_win32_surface_create(hDC);
-          cairo_status_t status = cairo_surface_status(window);
-          
-          // Check for error...
-          if (status != CAIRO_STATUS_SUCCESS)
-            {
-              // Output the surface create error...
-              NSWarnMLog(@"surface create FAILED - status: %s\n",  cairo_status_to_string(status));
+	{
+	  // Create the cairo surfaces...
+	  // NSBackingStoreRetained works like Buffered since 10.5 (See apple docs)...
+	  // NOTE: According to Apple docs NSBackingStoreBuffered should be the only one
+	  //       ever used anymore....others are NOT recommended...
+	  if (win && (win->type == NSBackingStoreNonretained))
+	    {
+	      // This is the raw DC surface...
+	      _surface = cairo_win32_surface_create(hDC);
+
+	      // Check for error...
+	      if (cairo_surface_status(_surface) != CAIRO_STATUS_SUCCESS)
+		{
+		  // Output the surface create error...
+		  cairo_status_t status = cairo_surface_status(_surface);
+		  NSWarnMLog(@"surface create FAILED - status: %s\n", cairo_status_to_string(status));
               
-              // And deallocate ourselves...
-              DESTROY(self);
-            }
-          else
-            {
-              // and this is the in-memory DC surface...surface owns its DC...
-              // NOTE: For some reason we get an init sequence with zero width/height,
-              //       which creates problems in the cairo layer.  It tries to clear
-              //       the 'similar' surface it's creating, and with a zero width/height
-              //       it incorrectly thinks the clear failed...so we will init with
-              //       a minimum size of 1 for width/height...
-              _surface = cairo_surface_create_similar(window, CAIRO_CONTENT_COLOR_ALPHA,
-                                                      MAX(1, csize.width),
-                                                      MAX(1, csize.height));
-              status = cairo_surface_status(_surface);
+		  // Destroy the initial surface created...
+		  cairo_surface_destroy(_surface);
+              
+		  // And deallocate ourselves...
+		  DESTROY(self);
 
-              // Check for error...
-              if (status != CAIRO_STATUS_SUCCESS)
-                {
-                  // Output the surface create error...
-                  NSWarnMLog(@"surface create FAILED - status: %s\n",  cairo_status_to_string(status));
-                  
-                  // Destroy the surface created...
-                  cairo_surface_destroy(_surface);
-                  
-                  // And deallocate ourselves...
-                  DESTROY(self);
-                }
-            }
+		  // Release the device context...
+                  ReleaseDC(device, hDC);
+		}
+	    }
+	  else
+	    {
+	      NSSize csize = [self size];
           
-          // Destroy the initial surface created...
-          cairo_surface_destroy(window);
-        }
-          
-      // Release the device context...
-      ReleaseDC((HWND)device, hDC);
+	      // This is the raw DC surface...
+	      cairo_surface_t *window = cairo_win32_surface_create(hDC);
 
-      if (win && self)
-        {
-          // We need this for handleExposeEvent in WIN32Server...
-          win->surface = (void*)self;
-        }
+	      // Check for error...
+	      if (cairo_surface_status(window) != CAIRO_STATUS_SUCCESS)
+		{
+		  // Output the surface create error...
+		  cairo_status_t status = cairo_surface_status(window);
+		  NSWarnMLog(@"surface create FAILED - status: %s\n",  cairo_status_to_string(status));
+                  
+		  // And deallocate ourselves...
+		  DESTROY(self);
+		}
+	      else
+		{
+		  // and this is the in-memory DC surface...surface owns its DC...
+		  // NOTE: For some reason we get an init sequence with zero width/height,
+		  //       which creates problems in the cairo layer.  It tries to clear
+		  //       the 'similar' surface it's creating, and with a zero width/height
+		  //       it incorrectly thinks the clear failed...so we will init with
+		  //       a minimum size of 1 for width/height...
+		  _surface = cairo_surface_create_similar(window, CAIRO_CONTENT_COLOR_ALPHA,
+							  MAX(1, csize.width),
+							  MAX(1, csize.height));
+		  cairo_status_t status = cairo_surface_status(_surface);
+
+		  // Check for error...
+		  if (status != CAIRO_STATUS_SUCCESS)
+		    {
+		      // Output the surface create error...
+		      NSWarnMLog(@"surface create FAILED - status: %s\n",  cairo_status_to_string(status));
+                  
+		      // Destroy the surface created...
+		      cairo_surface_destroy(_surface);
+                  
+		      // And deallocate ourselves...
+		      DESTROY(self);
+		    }
+		}
+          
+	      // Destroy the initial surface created...
+	      cairo_surface_destroy(window);
+
+	      // Release the device context...
+	      ReleaseDC((HWND)device, hDC);
+	    }
+          
+	  if (win && self)
+	    {
+	      // We need this for handleExposeEvent in WIN32Server...
+	      win->surface = (void*)self;
+	    }
+	}
     }
 
   return self;
