@@ -49,6 +49,7 @@
 #include <AppKit/NSTextField.h>
 #include <AppKit/DPSOperators.h>
 #include <GNUstepGUI/GSTheme.h>
+#include <GNUstepGUI/GSTrackingRect.h>
 
 #include "win32/WIN32Server.h"
 #include "win32/WIN32Geometry.h"
@@ -62,6 +63,12 @@
 
 #include <math.h>
 
+// To update the cursor..
+static BOOL update_cursor = NO;
+static BOOL should_handle_cursor = NO;
+static NSCursor *current_cursor = nil;
+
+
 // Forward declarations...
 static unsigned int mask_for_keystate(BYTE *keyState);
 
@@ -143,7 +150,7 @@ BOOL CALLBACK LoadDisplayMonitorInfo(HMONITOR hMonitor,
   NSMutableArray        *monitors = (NSMutableArray*)dwData;
   W32DisplayMonitorInfo *info = [[W32DisplayMonitorInfo alloc] initWithHMonitor:hMonitor rect:lprcMonitor];
   
-  NSLog(@"screen %ld:hdc: %ld frame:top:%ld left:%ld right:%ld bottom:%ld  frame:x:%f y:%f w:%f h:%f\n",
+  NSDebugLog(@"screen %ld:hdc: %ld frame:top:%ld left:%ld right:%ld bottom:%ld  frame:x:%f y:%f w:%f h:%f\n",
         [monitors count], (long)hMonitor,
         lprcMonitor->top, lprcMonitor->left,
         lprcMonitor->right, lprcMonitor->bottom,
@@ -1139,6 +1146,26 @@ LRESULT CALLBACK windowEnumCallback(HWND hwnd, LPARAM lParam)
  
   switch (uMsg)
     {
+      case WM_MOUSELEAVE:
+	{
+	  /* If the cursor leave the window remove the GNUstep cursors, send
+	   * the appropriate message and tell GNUstep stop handling
+	   * the cursor.
+	   */
+	  NSEvent *e;
+	  e = [NSEvent otherEventWithType: NSAppKitDefined
+				 location: NSMakePoint(-1,-1)
+			    modifierFlags: 0
+				timestamp: 0
+			     windowNumber: (int)hwnd
+				  context: GSCurrentContext()
+				  subtype: GSAppKitWindowLeave
+				    data1: 0
+				    data2: 0];
+	  [GSCurrentServer() postEvent: e atStart: YES];
+	  should_handle_cursor = NO;
+	}
+	break;
       case WM_SIZING:
         return [self decodeWM_SIZINGParams: hwnd : wParam : lParam];
         break;
@@ -1211,7 +1238,15 @@ LRESULT CALLBACK windowEnumCallback(HWND hwnd, LPARAM lParam)
         else
           [self decodeWM_KILLFOCUSParams: wParam : lParam : hwnd]; 
         break;
-      case WM_SETCURSOR: 
+      case WM_SETCURSOR:
+	if (wParam == (int)hwnd)
+	  {
+	    // Check if GNUstep should handle the cursor.
+	    if (should_handle_cursor)
+	      {
+		flags._eventHandled = YES;
+	      }
+	  }
         break;
       case WM_QUERYOPEN: 
         [self decodeWM_QUERYOPENParams: wParam : lParam : hwnd]; 
@@ -1241,9 +1276,29 @@ LRESULT CALLBACK windowEnumCallback(HWND hwnd, LPARAM lParam)
         break;
       case WM_APP: 
         break;  
-      case WM_ENTERMENULOOP: 
-        break;
-      case WM_EXITMENULOOP: 
+      case WM_ENTERMENULOOP:
+	/* If the user open a native contextual menu (a non GNUstep window)
+	 * send the appropriate message and tell GNUstep stop handling
+	 * the cursor.
+	 */
+	if (wParam)
+	  {
+	    NSEvent *e;
+	    [GSWindowWithNumber((int)hwnd) resetCursorRects];
+	    e = [NSEvent otherEventWithType: NSAppKitDefined
+				   location: NSMakePoint(-1,-1)
+			      modifierFlags: 0
+				  timestamp: 0
+			       windowNumber: (int)hwnd
+				    context: GSCurrentContext()
+				    subtype: GSAppKitWindowLeave
+				    data1: 0
+				      data2: 0];
+	    [GSCurrentServer() postEvent: e atStart: YES];
+	    should_handle_cursor = NO;
+	  }
+	break;
+      case WM_EXITMENULOOP:
         break;
       case WM_INITMENU: 
         break;
@@ -1288,8 +1343,8 @@ LRESULT CALLBACK windowEnumCallback(HWND hwnd, LPARAM lParam)
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "NCHITTEST", hwnd);
         break;
       case WM_NCMOUSEMOVE: //MOUSE
-        NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "NCMOUSEMOVE", hwnd);
-        break;
+	NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "NCMOUSEMOVE", hwnd);
+	break;
       case WM_NCLBUTTONDOWN:  //MOUSE
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "NCLBUTTONDOWN", hwnd);
         break;
@@ -1705,6 +1760,21 @@ LRESULT CALLBACK windowEnumCallback(HWND hwnd, LPARAM lParam)
 
   NSDebugLLog(@"WTrace", @"windowdevice: %d", winNum);
   window = GSWindowWithNumber(winNum);
+
+  /* FIXME:
+   * The windows with autodisplay set to NO aren't displayed correctly on
+   * Windows, no matter the backing store type used. And trying to redisplay
+   * these windows here in the server not takes effect. So if the window
+   * have set autodisplay to NO, we change it to YES before create the window.
+   * This problem affects the tooltips, but this solution is different to
+   * the one used in the TestPlant branch. Because that solution involves
+   * changes in the side of GUI.
+   */
+  if (![window isAutodisplay])
+    {
+      [window setAutodisplay: YES];
+    }
+  
   GetClientRect((HWND)winNum, &rect);
   h = rect.bottom - rect.top;
   [self styleoffsets: &l : &r : &t : &b : [window styleMask]];
@@ -2397,7 +2467,7 @@ process_char(WPARAM wParam, unsigned *eventModifierFlags)
       case VK_F8: WIN_FUNCTIONKEY return NSF8FunctionKey;
       case VK_F9: WIN_FUNCTIONKEY return NSF9FunctionKey;
       case VK_F10: WIN_FUNCTIONKEY return NSF10FunctionKey;
-      case VK_F11: WIN_FUNCTIONKEY return NSF12FunctionKey;
+      case VK_F11: WIN_FUNCTIONKEY return NSF11FunctionKey;
       case VK_F12: WIN_FUNCTIONKEY return NSF12FunctionKey;
       case VK_F13: WIN_FUNCTIONKEY return NSF13FunctionKey;
       case VK_F14: WIN_FUNCTIONKEY return NSF14FunctionKey;
@@ -2698,6 +2768,118 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
     }
   // What about other modifiers?
 
+  /* Currently GNUstep only proccess events inside the windows (contentview).
+   * So we should check if this is the first movement inside the window.
+   * And should consider also the case when this is the last movement inside
+   * the window.
+   */
+  if (!should_handle_cursor)
+    {
+      /* If this is the first movement inside the window, tell GNUstep
+       * that should handle the cursor and that should check if the
+       * cursor needs be updated. 
+       */
+      should_handle_cursor = YES;
+      update_cursor = YES;
+
+      /* We also starts tracking the mouse, so we receive the
+       * message WM_MOUSELEAVE when the mouse leaves the client area.
+       */
+      TRACKMOUSEEVENT tme;
+      tme.cbSize = sizeof(tme);
+      tme.dwFlags = TME_LEAVE;
+      tme.hwndTrack = hwnd;
+      TrackMouseEvent(&tme);
+      
+      /* If there are a previous cursor available (maybe a cursor that
+       * represent a tool) set it as the cursor. If not, set an arrow
+       * cursor (this is necessary because if the cursor is updated to,
+       * for example, an I Beam cursor, there will not be a default cursor
+       * to display when the user moves the mouse over, for example, an
+       * scrollbar).
+       */
+      if (current_cursor != nil)
+	{
+	  [current_cursor set];
+	  current_cursor = nil;
+	}
+      else
+	{
+	  [[NSCursor arrowCursor] set];
+	}
+    }
+  else
+    {
+      /* If the cursor is not associated to a tracking rectangle, not in
+       * the push/pop stack, save this. We do this for the case when, for
+       * example, the user choose a tool in a Tools window which sets a
+       * cursor for the tool and this cursor should be preserved between
+       * different windows.
+       */
+      if ([NSCursor count] == 0 &&
+	  ![current_cursor isEqual: [NSCursor currentCursor]])
+	{
+	  ASSIGN(current_cursor, [NSCursor currentCursor]);
+	}
+    }
+
+  // Check if we need update the cursor.
+  if (update_cursor)
+    {
+      NSView *subview = nil;
+      NSWindow *gswin = GSWindowWithNumber((int)hwnd);
+
+      subview = [[gswin contentView] hitTest: eventLocation];
+      
+      if (subview != nil && subview->_rFlags.valid_rects)
+	{
+	  NSArray *tr = subview->_cursor_rects;
+	  NSUInteger count = [tr count];
+
+	  // Loop through cursor rectangles
+	  if (count > 0)
+	    {
+	      GSTrackingRect *rects[count];
+	      NSUInteger i;
+
+	      [tr getObjects: rects];
+
+	      for (i = 0; i < count; ++i)
+		{
+		  GSTrackingRect *r = rects[i];
+		  BOOL now;
+
+		  if ([r isValid] == NO)
+		    continue;
+
+		  /*
+		   * Check for presence of point in rectangle.
+		   */
+		  now = NSMouseInRect(eventLocation, r->rectangle, NO);
+
+		  // Mouse inside
+		  if (now)
+		    {
+		      NSEvent *e;
+
+		      e = [NSEvent enterExitEventWithType: NSCursorUpdate
+						 location: eventLocation
+					    modifierFlags: eventFlags
+						timestamp: 0
+					     windowNumber: (int)hwnd
+						  context: gcontext
+					      eventNumber: 0
+					   trackingNumber: (int)YES
+						 userData: (void*)r];
+		      [GSCurrentServer() postEvent: e atStart: YES];
+		      //NSLog(@"Add enter event %@ for view %@ rect %@", e, theView, NSStringFromRect(r->rectangle));
+		    }
+		}
+	    }
+	}
+      update_cursor = NO;
+    }
+
   if (eventType == NSScrollWheel)
     {
       float delta = GET_WHEEL_DELTA_WPARAM(wParam) / 120.0;
